# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::Authorization::Mgmt::V2018_07_01_preview
  #
  # Role based access control provides you a way to apply granular level policy
  # administration down to individual resources or resource groups. These
  # operations enable you to get deny assignments. A deny assignment describes
  # the set of actions on resources that are denied for Azure Active Directory
  # users.
  #
  class DenyAssignments
    include MsRestAzure

    #
    # Creates and initializes a new instance of the DenyAssignments class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [AuthorizationManagementClient] reference to the AuthorizationManagementClient
    attr_reader :client

    #
    # Gets deny assignments for a resource.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param resource_provider_namespace [String] The namespace of the resource
    # provider.
    # @param parent_resource_path [String] The parent resource identity.
    # @param resource_type [String] The resource type of the resource.
    # @param resource_name [String] The name of the resource to get deny
    # assignments for.
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<DenyAssignment>] operation results.
    #
    def list_for_resource(resource_group_name, resource_provider_namespace, parent_resource_path, resource_type, resource_name, filter:nil, custom_headers:nil)
      first_page = list_for_resource_as_lazy(resource_group_name, resource_provider_namespace, parent_resource_path, resource_type, resource_name, filter:filter, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Gets deny assignments for a resource.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param resource_provider_namespace [String] The namespace of the resource
    # provider.
    # @param parent_resource_path [String] The parent resource identity.
    # @param resource_type [String] The resource type of the resource.
    # @param resource_name [String] The name of the resource to get deny
    # assignments for.
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_for_resource_with_http_info(resource_group_name, resource_provider_namespace, parent_resource_path, resource_type, resource_name, filter:nil, custom_headers:nil)
      list_for_resource_async(resource_group_name, resource_provider_namespace, parent_resource_path, resource_type, resource_name, filter:filter, custom_headers:custom_headers).value!
    end

    #
    # Gets deny assignments for a resource.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param resource_provider_namespace [String] The namespace of the resource
    # provider.
    # @param parent_resource_path [String] The parent resource identity.
    # @param resource_type [String] The resource type of the resource.
    # @param resource_name [String] The name of the resource to get deny
    # assignments for.
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_for_resource_async(resource_group_name, resource_provider_namespace, parent_resource_path, resource_type, resource_name, filter:nil, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, 'resource_provider_namespace is nil' if resource_provider_namespace.nil?
      fail ArgumentError, 'parent_resource_path is nil' if parent_resource_path.nil?
      fail ArgumentError, 'resource_type is nil' if resource_type.nil?
      fail ArgumentError, 'resource_name is nil' if resource_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourcegroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{parentResourcePath}/{resourceType}/{resourceName}/providers/Microsoft.Authorization/denyAssignments'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'resourceProviderNamespace' => resource_provider_namespace,'resourceName' => resource_name},
          skip_encoding_path_params: {'parentResourcePath' => parent_resource_path,'resourceType' => resource_type},
          query_params: {'api-version' => @client.api_version,'$filter' => filter},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Authorization::Mgmt::V2018_07_01_preview::Models::DenyAssignmentListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets deny assignments for a resource group.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<DenyAssignment>] operation results.
    #
    def list_for_resource_group(resource_group_name, filter:nil, custom_headers:nil)
      first_page = list_for_resource_group_as_lazy(resource_group_name, filter:filter, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Gets deny assignments for a resource group.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_for_resource_group_with_http_info(resource_group_name, filter:nil, custom_headers:nil)
      list_for_resource_group_async(resource_group_name, filter:filter, custom_headers:custom_headers).value!
    end

    #
    # Gets deny assignments for a resource group.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_for_resource_group_async(resource_group_name, filter:nil, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Authorization/denyAssignments'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name},
          query_params: {'api-version' => @client.api_version,'$filter' => filter},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Authorization::Mgmt::V2018_07_01_preview::Models::DenyAssignmentListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets all deny assignments for the subscription.
    #
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<DenyAssignment>] operation results.
    #
    def list(filter:nil, custom_headers:nil)
      first_page = list_as_lazy(filter:filter, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Gets all deny assignments for the subscription.
    #
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_with_http_info(filter:nil, custom_headers:nil)
      list_async(filter:filter, custom_headers:custom_headers).value!
    end

    #
    # Gets all deny assignments for the subscription.
    #
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_async(filter:nil, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/providers/Microsoft.Authorization/denyAssignments'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version,'$filter' => filter},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Authorization::Mgmt::V2018_07_01_preview::Models::DenyAssignmentListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Get the specified deny assignment.
    #
    # @param scope [String] The scope of the deny assignment.
    # @param deny_assignment_id [String] The ID of the deny assignment to get.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DenyAssignment] operation results.
    #
    def get(scope, deny_assignment_id, custom_headers:nil)
      response = get_async(scope, deny_assignment_id, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Get the specified deny assignment.
    #
    # @param scope [String] The scope of the deny assignment.
    # @param deny_assignment_id [String] The ID of the deny assignment to get.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(scope, deny_assignment_id, custom_headers:nil)
      get_async(scope, deny_assignment_id, custom_headers:custom_headers).value!
    end

    #
    # Get the specified deny assignment.
    #
    # @param scope [String] The scope of the deny assignment.
    # @param deny_assignment_id [String] The ID of the deny assignment to get.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(scope, deny_assignment_id, custom_headers:nil)
      fail ArgumentError, 'scope is nil' if scope.nil?
      fail ArgumentError, 'deny_assignment_id is nil' if deny_assignment_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{scope}/providers/Microsoft.Authorization/denyAssignments/{denyAssignmentId}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'denyAssignmentId' => deny_assignment_id},
          skip_encoding_path_params: {'scope' => scope},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Authorization::Mgmt::V2018_07_01_preview::Models::DenyAssignment.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets a deny assignment by ID.
    #
    # @param deny_assignment_id [String] The fully qualified deny assignment ID.
    # For example, use the format,
    # /subscriptions/{guid}/providers/Microsoft.Authorization/denyAssignments/{denyAssignmentId}
    # for subscription level deny assignments, or
    # /providers/Microsoft.Authorization/denyAssignments/{denyAssignmentId} for
    # tenant level deny assignments.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DenyAssignment] operation results.
    #
    def get_by_id(deny_assignment_id, custom_headers:nil)
      response = get_by_id_async(deny_assignment_id, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a deny assignment by ID.
    #
    # @param deny_assignment_id [String] The fully qualified deny assignment ID.
    # For example, use the format,
    # /subscriptions/{guid}/providers/Microsoft.Authorization/denyAssignments/{denyAssignmentId}
    # for subscription level deny assignments, or
    # /providers/Microsoft.Authorization/denyAssignments/{denyAssignmentId} for
    # tenant level deny assignments.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_by_id_with_http_info(deny_assignment_id, custom_headers:nil)
      get_by_id_async(deny_assignment_id, custom_headers:custom_headers).value!
    end

    #
    # Gets a deny assignment by ID.
    #
    # @param deny_assignment_id [String] The fully qualified deny assignment ID.
    # For example, use the format,
    # /subscriptions/{guid}/providers/Microsoft.Authorization/denyAssignments/{denyAssignmentId}
    # for subscription level deny assignments, or
    # /providers/Microsoft.Authorization/denyAssignments/{denyAssignmentId} for
    # tenant level deny assignments.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_by_id_async(deny_assignment_id, custom_headers:nil)
      fail ArgumentError, 'deny_assignment_id is nil' if deny_assignment_id.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{denyAssignmentId}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'denyAssignmentId' => deny_assignment_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Authorization::Mgmt::V2018_07_01_preview::Models::DenyAssignment.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets deny assignments for a scope.
    #
    # @param scope [String] The scope of the deny assignments.
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<DenyAssignment>] operation results.
    #
    def list_for_scope(scope, filter:nil, custom_headers:nil)
      first_page = list_for_scope_as_lazy(scope, filter:filter, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Gets deny assignments for a scope.
    #
    # @param scope [String] The scope of the deny assignments.
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_for_scope_with_http_info(scope, filter:nil, custom_headers:nil)
      list_for_scope_async(scope, filter:filter, custom_headers:custom_headers).value!
    end

    #
    # Gets deny assignments for a scope.
    #
    # @param scope [String] The scope of the deny assignments.
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_for_scope_async(scope, filter:nil, custom_headers:nil)
      fail ArgumentError, 'scope is nil' if scope.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{scope}/providers/Microsoft.Authorization/denyAssignments'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'scope' => scope},
          query_params: {'api-version' => @client.api_version,'$filter' => filter},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Authorization::Mgmt::V2018_07_01_preview::Models::DenyAssignmentListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets deny assignments for a resource.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DenyAssignmentListResult] operation results.
    #
    def list_for_resource_next(next_page_link, custom_headers:nil)
      response = list_for_resource_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets deny assignments for a resource.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_for_resource_next_with_http_info(next_page_link, custom_headers:nil)
      list_for_resource_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Gets deny assignments for a resource.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_for_resource_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Authorization::Mgmt::V2018_07_01_preview::Models::DenyAssignmentListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets deny assignments for a resource group.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DenyAssignmentListResult] operation results.
    #
    def list_for_resource_group_next(next_page_link, custom_headers:nil)
      response = list_for_resource_group_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets deny assignments for a resource group.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_for_resource_group_next_with_http_info(next_page_link, custom_headers:nil)
      list_for_resource_group_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Gets deny assignments for a resource group.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_for_resource_group_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Authorization::Mgmt::V2018_07_01_preview::Models::DenyAssignmentListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets all deny assignments for the subscription.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DenyAssignmentListResult] operation results.
    #
    def list_next(next_page_link, custom_headers:nil)
      response = list_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets all deny assignments for the subscription.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_next_with_http_info(next_page_link, custom_headers:nil)
      list_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Gets all deny assignments for the subscription.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Authorization::Mgmt::V2018_07_01_preview::Models::DenyAssignmentListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets deny assignments for a scope.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DenyAssignmentListResult] operation results.
    #
    def list_for_scope_next(next_page_link, custom_headers:nil)
      response = list_for_scope_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets deny assignments for a scope.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_for_scope_next_with_http_info(next_page_link, custom_headers:nil)
      list_for_scope_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Gets deny assignments for a scope.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_for_scope_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::Authorization::Mgmt::V2018_07_01_preview::Models::DenyAssignmentListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets deny assignments for a resource.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param resource_provider_namespace [String] The namespace of the resource
    # provider.
    # @param parent_resource_path [String] The parent resource identity.
    # @param resource_type [String] The resource type of the resource.
    # @param resource_name [String] The name of the resource to get deny
    # assignments for.
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DenyAssignmentListResult] which provide lazy access to pages of the
    # response.
    #
    def list_for_resource_as_lazy(resource_group_name, resource_provider_namespace, parent_resource_path, resource_type, resource_name, filter:nil, custom_headers:nil)
      response = list_for_resource_async(resource_group_name, resource_provider_namespace, parent_resource_path, resource_type, resource_name, filter:filter, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_for_resource_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # Gets deny assignments for a resource group.
    #
    # @param resource_group_name [String] The name of the resource group.
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DenyAssignmentListResult] which provide lazy access to pages of the
    # response.
    #
    def list_for_resource_group_as_lazy(resource_group_name, filter:nil, custom_headers:nil)
      response = list_for_resource_group_async(resource_group_name, filter:filter, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_for_resource_group_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # Gets all deny assignments for the subscription.
    #
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DenyAssignmentListResult] which provide lazy access to pages of the
    # response.
    #
    def list_as_lazy(filter:nil, custom_headers:nil)
      response = list_async(filter:filter, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # Gets deny assignments for a scope.
    #
    # @param scope [String] The scope of the deny assignments.
    # @param filter [String] The filter to apply on the operation. Use
    # $filter=atScope() to return all deny assignments at or above the scope. Use
    # $filter=denyAssignmentName eq '{name}' to search deny assignments by name at
    # specified scope. Use $filter=principalId eq '{id}' to return all deny
    # assignments at, above and below the scope for the specified principal. Use
    # $filter=gdprExportPrincipalId eq '{id}' to return all deny assignments at,
    # above and below the scope for the specified principal. This filter is
    # different from the principalId filter as it returns not only those deny
    # assignments that contain the specified principal is the Principals list but
    # also those deny assignments that contain the specified principal is the
    # ExcludePrincipals list. Additionally, when gdprExportPrincipalId filter is
    # used, only the deny assignment name and description properties are returned.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DenyAssignmentListResult] which provide lazy access to pages of the
    # response.
    #
    def list_for_scope_as_lazy(scope, filter:nil, custom_headers:nil)
      response = list_for_scope_async(scope, filter:filter, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_for_scope_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

  end
end
